@@ -1,9 +1,11 @@
 # -*- coding: utf-8 -*-
 from __future__ import unicode_literals
 
-from django.db import migrations, models
+from django.db import migrations
 from django.conf import settings
 
+from simple_log.fields import SimpleManyToManyField
+
 
 class Migration(migrations.Migration):
 
@@ -16,6 +18,6 @@ class Migration(migrations.Migration):
         migrations.AddField(
             model_name='simplelog',
             name='related_logs',
-            field=models.ManyToManyField(to=settings.SIMPLE_LOG_MODEL, verbose_name='related log', blank=True),
+            field=SimpleManyToManyField(to='self', verbose_name='related log', blank=True),
         ),
     ]
