@@ -1,36 +1,39 @@
 # -*- coding: utf-8 -*-
 from __future__ import absolute_import, unicode_literals
 
+import datetime
+import os
+
+from request_vars.utils import del_variable, get_variable, set_variable
+
 from django.conf import settings as django_settings
 from django.contrib.admin.utils import quote
 from django.contrib.contenttypes.models import ContentType
 from django.core.exceptions import ValidationError
 from django.core.validators import validate_ipv46_address
-from django.db import models
+from django.db import connection, models
+from django.urls import NoReverseMatch, reverse
 from django.utils import timezone
 from django.utils.encoding import force_text, python_2_unicode_compatible
 from django.utils.functional import cached_property
 from django.utils.translation import ugettext_lazy as _
 
+from simple_log.fields import SimpleJSONField, SimpleManyToManyField
+from simple_log.signals import save_logs_on_commit
+
 from .conf import settings
-from .utils import get_current_request, get_current_user, get_fields
-
-try:
-    from django.urls import reverse, NoReverseMatch
-except ImportError:
-    from django.core.urlresolvers import reverse, NoReverseMatch
-
-try:
-    from django.contrib.postgres.fields.jsonb import JSONField
-except ImportError:
-    from jsonfield import JSONField
+from .utils import (
+    get_current_request, get_current_user, get_fields, get_obj_repr,
+    get_serializer
+)
 
 
-__all__ = ['SimpleLogAbstract', 'SimpleLog', 'ModelSerializer']
+__all__ = ['SimpleLogAbstractBase', 'SimpleLogAbstract', 'SimpleLog',
+           'ModelSerializer']
 
 
 @python_2_unicode_compatible
-class SimpleLogAbstract(models.Model):
+class SimpleLogAbstractBase(models.Model):
     ADD = 1
     CHANGE = 2
     DELETE = 3
@@ -57,21 +60,19 @@ class SimpleLogAbstract(models.Model):
         verbose_name=_('user'),
         null=True
     )
-    user_repr = models.CharField(_('user repr'), blank=True, max_length=1000)
+    user_repr = models.TextField(_('user repr'), blank=True)
     user_ip = models.GenericIPAddressField(_('IP address'), null=True)
     object_id = models.TextField(_('object id'), blank=True, null=True)
-    object_repr = models.CharField(_('object repr'), max_length=1000)
-    action_flag = models.PositiveSmallIntegerField(
-        _('action flag'),
-        choices=ACTION_CHOICES
-    )
-    old = JSONField(_('old values'), null=True)
-    new = JSONField(_('new values'), null=True)
+    object_repr = models.TextField(_('object repr'), blank=True)
+    old = SimpleJSONField(_('old values'), null=True)
+    new = SimpleJSONField(_('new values'), null=True)
+    change_message = models.TextField(_('change message'), blank=True)
 
-    related_logs = models.ManyToManyField(
-        django_settings.SIMPLE_LOG_MODEL,
+    related_logs = SimpleManyToManyField(
+        'self',
         verbose_name=_('related log'),
         blank=True,
+        symmetrical=False,
     )
 
     is_add = property(lambda self: self.action_flag == self.ADD)
@@ -85,7 +86,16 @@ class SimpleLogAbstract(models.Model):
         abstract = True
 
     def __str__(self):
-        return '%s: %s' % (self.object_repr, self.get_action_flag_display())
+        return '%s' % self.object_repr
+
+    def save(self, *args, **kwargs):
+        if settings.SAVE_ONLY_CHANGED:
+            changed = self.changed_fields.keys()
+            self.old = {k: v for k, v in (self.old or {}).items()
+                        if k in changed} or None
+            self.new = {k: v for k, v in (self.new or {}).items()
+                        if k in changed} or None
+        super(SimpleLogAbstractBase, self).save(*args, **kwargs)
 
     def get_edited_object(self):
         return self.content_type.get_object_for_this_type(pk=self.object_id)
@@ -101,21 +111,13 @@ class SimpleLogAbstract(models.Model):
         return None
 
     @classmethod
-    def log(cls, instance, commit=True, **kwargs):
-        user = kwargs.get('user')
-        if 'user' not in kwargs:
+    def get_log_params(cls, instance, **kwargs):
+        if 'user' in kwargs:
+            user = kwargs['user']
+        else:
             user = get_current_user()
-        if 'user_repr' not in kwargs:
-            if user is None:
-                kwargs['user_repr'] = settings.NONE_USER_REPR
-            elif user.is_authenticated():
-                kwargs['user_repr'] = force_text(user)
-            else:
-                kwargs['user_repr'] = settings.ANONYMOUS_REPR
-        if 'user_ip' not in kwargs:
-            kwargs['user_ip'] = cls.get_ip()
-        kwargs.update({
-            'content_type': ContentType.objects.get_for_model(
+        params = dict(
+            content_type=ContentType.objects.get_for_model(
                 instance.__class__,
                 for_concrete_model=getattr(
                     instance,
@@ -123,13 +125,61 @@ class SimpleLogAbstract(models.Model):
                     settings.PROXY_CONCRETE
                 )
             ),
-            'object_id': instance.pk,
-            'object_repr': force_text(instance),
-            'user': user if user and user.is_authenticated() else None
-        })
-        obj = cls(**kwargs)
+            object_id=instance.pk,
+            object_repr=get_obj_repr(instance),
+            user=user if user and user.is_authenticated else None,
+            user_repr=cls.get_user_repr(user),
+            user_ip=cls.get_ip()
+        )
+        params.update(getattr(instance, 'simple_log_params', {}))
+        params.update(kwargs)
+        return params
+
+    @classmethod
+    def add_to_thread(cls, obj):
+        in_commit = 'sl_in_commit' in \
+                    [f[1].__name__ for f in connection.run_on_commit]
+        logs = get_variable('logs', [])
+        # prevent memory usage in non transaction test cases
+        if not in_commit and logs:
+            del_variable('logs')
+            logs = []
+        logs.append(obj)
+        set_variable('logs', logs)
+
+        if not in_commit:
+            def sl_in_commit():
+                save_logs_on_commit(logs)
+            connection.on_commit(sl_in_commit)
+
+    @classmethod
+    def set_initial(cls, instance):
+        if instance.pk and \
+                not hasattr(instance, settings.OLD_INSTANCE_ATTR_NAME):
+            setattr(
+                instance,
+                settings.OLD_INSTANCE_ATTR_NAME,
+                instance.__class__.objects.filter(pk=instance.pk)
+                                          .select_related().first()
+            )
+        if not hasattr(instance, '_old_values'):
+            serializer = get_serializer(instance.__class__)()
+            instance._old_values = serializer(
+                getattr(instance, settings.OLD_INSTANCE_ATTR_NAME, None)
+            )
+
+    @classmethod
+    def log(cls, instance, commit=True, with_initial=False,
+            force_save=False, **kwargs):
+        if with_initial:
+            cls.set_initial(instance)
+        obj = cls(**cls.get_log_params(instance, **kwargs))
+        obj.force_save = force_save
+        obj.instance = instance
+        obj.disable_related = get_variable('disable_related', False)
         if commit:
             obj.save()
+        cls.add_to_thread(obj)
         return obj
 
     @cached_property
@@ -169,6 +219,15 @@ class SimpleLogAbstract(models.Model):
             except ValidationError:
                 pass
 
+    @classmethod
+    def get_user_repr(cls, user):
+        if user is None:
+            return settings.NONE_USER_REPR
+        elif user.is_authenticated:
+            return force_text(user)
+        else:
+            return settings.ANONYMOUS_REPR
+
     def get_differences(self):
         old = self.old or {}
         new = self.new or {}
@@ -179,22 +238,35 @@ class SimpleLogAbstract(models.Model):
         } for key, value in self.changed_fields.items()]
 
 
+class SimpleLogAbstract(SimpleLogAbstractBase):
+    action_flag = models.PositiveSmallIntegerField(
+        _('action flag'),
+        choices=SimpleLogAbstractBase.ACTION_CHOICES
+    )
+
+    class Meta(SimpleLogAbstractBase.Meta):
+        abstract = True
+
+    def __str__(self):
+        return '%s: %s' % (self.object_repr, self.get_action_flag_display())
+
+
 class SimpleLog(SimpleLogAbstract):
     class Meta(SimpleLogAbstract.Meta):
         swappable = 'SIMPLE_LOG_MODEL'
 
 
 class ModelSerializer(object):
-    def __call__(self, instance, override=None):
-        return self.serialize(instance, override)
+    def __call__(self, instance):
+        return self.serialize(instance)
 
-    def serialize(self, instance, override=None):
-        if not instance:
+    def serialize(self, instance):
+        if not (instance and instance.pk):
             return None
         return {
             field.name: {
                 'label': self.get_field_label(field),
-                'value': self.get_field_value(instance, field, override)
+                'value': self.get_field_value(instance, field)
             } for field in get_fields(instance.__class__)
         }
 
@@ -203,9 +275,7 @@ class ModelSerializer(object):
             return force_text(field.related_model._meta.verbose_name_plural)
         return force_text(field.verbose_name)
 
-    def get_field_value(self, instance, field, override=None):
-        if override and field.name in override:
-            return self.get_value_for_type(override[field.name])
+    def get_field_value(self, instance, field):
         if field.many_to_many:
             return self.get_m2m_value(instance, field)
         elif field.one_to_many:
@@ -214,18 +284,20 @@ class ModelSerializer(object):
             return self.get_fk_value(instance, field)
         elif getattr(field, 'choices', None):
             return self.get_choice_value(instance, field)
+        elif isinstance(field, models.FileField):
+            return self.get_file_value(instance, field)
         return self.get_other_value(instance, field)
 
     def get_m2m_value(self, instance, field):
         return [{
             'db': self.get_value_for_type(x.pk),
-            'repr': force_text(x)
+            'repr': get_obj_repr(x)
         } for x in getattr(instance, field.name).iterator()]
 
     def get_o2m_value(self, instance, field):
         return [{
             'db': self.get_value_for_type(x.pk),
-            'repr': force_text(x)
+            'repr': get_obj_repr(x)
         } for x in getattr(instance, field.name).iterator()]
 
     def get_fk_value(self, instance, field):
@@ -244,6 +316,12 @@ class ModelSerializer(object):
             ) or '',
         }
 
+    def get_file_value(self, instance, field):
+        value = self.get_value_for_type(field.value_from_object(instance))
+        if settings.FILE_NAME_ONLY or instance.simple_log_file_name_only:
+            value = os.path.basename(value)
+        return value
+
     def get_other_value(self, instance, field):
         return self.get_value_for_type(field.value_from_object(instance))
 
@@ -251,4 +329,10 @@ class ModelSerializer(object):
     def get_value_for_type(value):
         if value is None or isinstance(value, (int, bool, dict, list)):
             return value
+        if isinstance(value, datetime.datetime) and settings.DATETIME_FORMAT:
+            return value.strftime(settings.DATETIME_FORMAT)
+        if isinstance(value, datetime.date) and settings.DATE_FORMAT:
+            return value.strftime(settings.DATE_FORMAT)
+        if isinstance(value, datetime.time) and settings.TIME_FORMAT:
+            return value.strftime(settings.TIME_FORMAT)
         return force_text(value)
